require 'webmock/rspec'
require 'json'
require 'crabbs/crawler'

describe Crabbs::Crawler do
  describe '#crawl' do
    let(:options) { Hash.new }
    subject { Crabbs::Crawler.new options }

    context 'when verbose' do
      let(:options) { { verbose: true } }

      before do
        @uri_string = 'http://example.com/'
        stub_request(:get, @uri_string).to_return(body: %Q{<a href="/path"></a><a href="/local"></a><a href="http://fb.com/"></a>})
        stub_request(:get, "http://example.com/path").to_return(body: "")
        stub_request(:get, "http://example.com/local").to_return(body: "")
      end

      it 'logs the url been visited' do
        allow(STDOUT).to receive(:puts)

        subject.crawl @uri_string

        expect(STDOUT).to have_received(:puts).with('Visiting: http://example.com/')
        expect(STDOUT).to have_received(:puts).with('Visiting: http://example.com/path')
        expect(STDOUT).to have_received(:puts).with('Visiting: http://example.com/local')
      end
    end

    context 'when not verbose' do
      let(:options) { { verbose: false } }

      before do
        @uri_string = 'http://example.com/'
        stub_request(:get, @uri_string).to_return(body: %Q{<a href="/path"></a><a href="/local"></a><a href="http://fb.com/"></a>})
        stub_request(:get, "http://example.com/path").to_return(body: "")
        stub_request(:get, "http://example.com/local").to_return(body: "")
      end

      it 'logs the url been visited' do
        allow(STDOUT).to receive(:putc)

        subject.crawl @uri_string

        expect(STDOUT).to have_received(:putc).with('.').exactly(3).times
      end
    end

    context 'an invalid URI' do
      it 'stores single entry site map' do
        subject.crawl %q{invalid\uri}

        subject.site_map.should == {"invalid\\uri"=>{}}
      end
    end

    context 'a valid URI' do
      context 'non html response' do
        before do
          @uri_string = 'http://example.com/test.json'
          stub_request(:get, @uri_string).to_return(body: { a: { href: '/test' } }.to_json, headers: { 'Content-Type' => 'application/json' })
        end

        it 'stores empty hash map' do
          subject.crawl @uri_string

          subject.site_map.should == { 'http://example.com/test.json' => {} }
        end
      end

      context 'without recursive links' do
        before do
          @uri_string = 'http://example.com/'
          stub_request(:get, @uri_string).to_return(body: %Q{<a href="/path"></a><a href="/local"></a><a href="http://fb.com/"></a>})
          stub_request(:get, "http://example.com/path").to_return(body: "")
          stub_request(:get, "http://example.com/local").to_return(body: "")
        end

        it 'stores the visted links' do
          subject.crawl @uri_string

          subject.visited.should include('http://example.com/')
        end

        it 'stores a hash with the site map' do
          subject.crawl @uri_string

          expected_site_map = {
            'http://example.com/' => {
              'http://example.com/path' => {},
              'http://example.com/local' => {}
            }
          }

          subject.site_map.should == expected_site_map
        end
      end

      context 'with recursive links' do
        before :each do
          @uri_string = 'http://example.com/'
          stub_request(:get, @uri_string).to_return(body: %Q{<a href="/path"></a>})
          stub_request(:get, "http://example.com/path").to_return(body: %Q{<a href="http://example.com/"></a>})
        end

        it 'visits both pages' do
          subject.crawl @uri_string

          subject.visited.should == ['http://example.com/', 'http://example.com/path']
        end

        it 'stores a hash with the site map' do
          subject.crawl @uri_string

          expected_site_map = {
            'http://example.com/' => {
              'http://example.com/path' => {
                'http://example.com/' => {}
              }
            }
          }

          subject.site_map.should == expected_site_map
        end
      end
    end
  end
end
