require 'crabbs/link'

describe Crabbs::Link do
  subject { Crabbs::Link.new href }

  describe '#same_host_as?' do
    context 'when it is the same host' do
      let(:host) { 'http://example.com/path' }
      let(:href) { 'http://example.com/path' }

      it 'is the same host' do
        subject.same_host_as?(host).should be_true
      end
    end

    context 'when href have no host' do
      let(:host) { 'http://example.com/path' }
      let(:href) { '/path' }

      it 'assumes its the same host' do
        subject.same_host_as?(host).should be_true
      end
    end

    context 'when host is a subdomain' do
      let(:host) { 'http://example.com/path' }
      let(:href) { 'http://subdomain.example.com/path' }

      it 'is not the same host' do
        subject.same_host_as?(host).should be_false
      end
    end

    context 'when host is completely different' do
      let(:host) { 'http://example.com/path' }
      let(:href) { 'http://facebook.com/path' }

      it 'is not the same host' do
        subject.same_host_as?(host).should be_false
      end
    end
  end

  describe '#has_valid_fragment?' do
    context 'when the fragment is empty' do
      let(:href) { 'http://example.com/path#' }

      it 'is not valid' do
        subject.has_valid_fragment?.should be_false
      end
    end

    context 'when there is no fragment' do
      let(:href) { 'http://example.com/path' }

      it 'is valid' do
        subject.has_valid_fragment?.should be_true
      end
    end

    context 'when there is a fragment' do
      let(:href) { 'http://example.com/path#fragment' }

      it 'is valid' do
        subject.has_valid_fragment?.should be_true
      end
    end
  end

  describe '#has_html_extension?' do
    context 'when the extension is not html' do
      let(:href) { 'http://example.com/path.zip' }

      it 'has no html extension' do
        subject.has_html_extension?.should be_false
      end
    end

    context 'when the extension html' do
      let(:href) { 'http://example.com/path.html' }

      it 'has html extension' do
        subject.has_html_extension?.should be_true
      end
    end

    context 'when there is no extension' do
      let(:href) { 'http://example.com/path' }

      it 'assumes to be a html link' do
        subject.has_html_extension?.should be_true
      end
    end
  end

  describe '#join' do
    context 'when the href is a root path' do
      let(:href) { '/path' }
      let(:url) { 'http://example.com/' }

      it 'joins with the url' do
        subject.join(url).should == 'http://example.com/path'
      end
    end

    context 'when the href is a direct path' do
      let(:href) { 'path' }
      let(:url) { 'http://example.com/test/' }

      it 'appends to the url previous path' do
        subject.join(url).should == 'http://example.com/test/path'
      end
    end

    context 'when the href is a fragment (hash)' do
      let(:href) { '#fragment' }
      let(:url) { 'http://example.com/test/' }

      it 'appends to the url previous path' do
        subject.join(url).should == 'http://example.com/test/#fragment'
      end
    end

    context 'when the href is a query parameter' do
      let(:href) { '?parameter' }
      let(:url) { 'http://example.com/test/' }

      it 'appends to the url previous path' do
        subject.join(url).should == 'http://example.com/test/?parameter'
      end
    end

    context 'when the href is both (fragment + query)' do
      let(:href) { '#fragment?parameter' }
      let(:url) { 'http://example.com/test/' }

      it 'appends to the url previous path' do
        subject.join(url).should == 'http://example.com/test/#fragment?parameter'
      end
    end

    context 'when the href is a full uri' do
      let(:href) { 'http://example.com/test/#fragment?parameter' }
      let(:url) { 'http://example.com/test/' }

      it 'leaves it untouched' do
        subject.join(url).should == 'http://example.com/test/#fragment?parameter'
      end
    end
  end
end
